@@ -3,6 +3,10 @@
 Supports the definition of commands in separate classes to be composed into cmd2.Cmd
 """
 from typing import (
+    TYPE_CHECKING,
+    Callable,
+    Dict,
+    Mapping,
     Optional,
     Type,
 )
@@ -14,18 +18,19 @@ from .constants import (
 from .exceptions import (
     CommandSetRegistrationError,
 )
+from .utils import (
+    Settable,
+)
 
-# Allows IDEs to resolve types without impacting imports at runtime, breaking circular dependency issues
-try:  # pragma: no cover
-    from typing import TYPE_CHECKING
-    if TYPE_CHECKING:
-        import cmd2
+if TYPE_CHECKING:  # pragma: no cover
+    import cmd2
 
-except ImportError:   # pragma: no cover
-    pass
+#: Callable signature for a basic command  function
+#: Further refinements are needed to define the input parameters
+CommandFunc = Callable[..., Optional[bool]]
 
 
-def with_default_category(category: str, *, heritable: bool = True):
+def with_default_category(category: str, *, heritable: bool = True) -> Callable[[Type['CommandSet']], Type['CommandSet']]:
     """
     Decorator that applies a category to all ``do_*`` command methods in a class that do not already
     have a category specified.
@@ -44,26 +49,35 @@ def with_default_category(category: str, *, heritable: bool = True):
     :return: decorator function
     """
 
-    def decorate_class(cls: Type[CommandSet]):
+    def decorate_class(cls: Type[CommandSet]) -> Type[CommandSet]:
         if heritable:
             setattr(cls, CLASS_ATTR_DEFAULT_HELP_CATEGORY, category)
 
-        from .constants import CMD_ATTR_HELP_CATEGORY
         import inspect
-        from .decorators import with_category
+
+        from .constants import (
+            CMD_ATTR_HELP_CATEGORY,
+        )
+        from .decorators import (
+            with_category,
+        )
+
         # get members of the class that meet the following criteria:
         # 1. Must be a function
         # 2. Must start with COMMAND_FUNC_PREFIX (do_)
         # 3. Must be a member of the class being decorated and not one inherited from a parent declaration
         methods = inspect.getmembers(
             cls,
-            predicate=lambda meth: inspect.isfunction(meth) and meth.__name__.startswith(COMMAND_FUNC_PREFIX)
-            and meth in inspect.getmro(cls)[0].__dict__.values())
+            predicate=lambda meth: inspect.isfunction(meth)
+            and meth.__name__.startswith(COMMAND_FUNC_PREFIX)
+            and meth in inspect.getmro(cls)[0].__dict__.values(),
+        )
         category_decorator = with_category(category)
         for method in methods:
             if not hasattr(method[1], CMD_ATTR_HELP_CATEGORY):
                 setattr(cls, method[0], category_decorator(method[1]))
         return cls
+
     return decorate_class
 
 
@@ -76,17 +90,18 @@ class CommandSet(object):
     ``do_``, ``help_``, and ``complete_`` functions differ only in that self is the CommandSet instead of the cmd2 app
     """
 
-    def __init__(self):
-        self._cmd = None  # type: Optional[cmd2.Cmd]
+    def __init__(self) -> None:
+        self._cmd: Optional[cmd2.Cmd] = None
+        self._settables: Dict[str, Settable] = {}
+        self._settable_prefix = self.__class__.__name__
 
-    def on_register(self, cmd) -> None:
+    def on_register(self, cmd: 'cmd2.Cmd') -> None:
         """
         Called by cmd2.Cmd as the first step to registering a CommandSet. The commands defined in this class have
         not be added to the CLI object at this point. Subclasses can override this to perform any initialization
         requiring access to the Cmd object (e.g. configure commands and their parsers based on CLI state data).
 
         :param cmd: The cmd2 main application
-        :type cmd: cmd2.Cmd
         """
         if self._cmd is None:
             self._cmd = cmd
@@ -114,3 +129,39 @@ class CommandSet(object):
         Subclasses can override this to perform remaining cleanup steps.
         """
         self._cmd = None
+
+    @property
+    def settable_prefix(self) -> str:
+        return self._settable_prefix
+
+    @property
+    def settables(self) -> Mapping[str, Settable]:
+        return self._settables
+
+    def add_settable(self, settable: Settable) -> None:
+        """
+        Convenience method to add a settable parameter to the CommandSet
+
+        :param settable: Settable object being added
+        """
+        if self._cmd:
+            if not self._cmd.always_prefix_settables:
+                if settable.name in self._cmd.settables.keys() and settable.name not in self._settables.keys():
+                    raise KeyError(f'Duplicate settable: {settable.name}')
+            else:
+                prefixed_name = f'{self._settable_prefix}.{settable.name}'
+                if prefixed_name in self._cmd.settables.keys() and settable.name not in self._settables.keys():
+                    raise KeyError(f'Duplicate settable: {settable.name}')
+        self._settables[settable.name] = settable
+
+    def remove_settable(self, name: str) -> None:
+        """
+        Convenience method for removing a settable parameter from the CommandSet
+
+        :param name: name of the settable being removed
+        :raises: KeyError if the Settable matches this name
+        """
+        try:
+            del self._settables[name]
+        except KeyError:
+            raise KeyError(name + " is not a settable parameter")
